-- When - a calendar
-- Copyright (C) 2023 Duane Robertson

-- main.lua

-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <https://www.gnu.org/licenses/>.


-- local dh = require 'duane_helper'


local when = require 'when'


local FILE_NAMES = {
	'calendar',
	'sample-calendar',
}
local FONT_FILE = 'art/roboto/Roboto-Medium.ttf'
local FSIZE = 45
local FUTURE = 365 -- 365.0 * 10.0
local IWIDTH = 720
local WRAP = IWIDTH - FSIZE * 2

local android
local canv
local canv_ht
local canv_off = 0
local font
local scale = { x = 1, y = 1 }
local time = 1
local whx, why = 0, 0
local whenq = {}
local when_text = {}


local function get_ht()
	local already = {}
	local ct = 1

	for _, v in ipairs(whenq) do
		if not already[v.text] then
			ct = ct + 1
			already[v.text] = true
		end
	end

	print(ct)
	return ct
end


local function limiter()
	if canv_off < - canv_ht then
		canv_off = - canv_ht
	end

	if canv_off > 0 then
		canv_off = 0
	end
end


local function parseCalendar()
	local ct = 0
	local items = {}
	local file_name

	for _, fn in ipairs(FILE_NAMES) do
		if love.filesystem.getInfo(fn) then
			file_name = fn
			break
		end
	end

	if not file_name then
		return
	end

	for lin in love.filesystem.lines(file_name) do
		ct = ct + 1
		local item = when:parseEntry(lin, ct)
		if item then
			items[#items+1] = item
		end
	end

	return items
end


local function print_cal()
	love.graphics.setCanvas(canv)
	love.graphics.clear()

	local already = {}

	when_text[1] = {0.8, 0.8, 0.8, 1}
	when_text[2] = os.date('%a %Y %b %e %H:%M:%S') .. '\n\n'

	if #when_text < 3 then
		for _, v in ipairs(whenq) do
			if not already[v.text] then
				local dat = v.date:gsub('^([^ ]+) *', '%1, '):gsub(' +', ' ')
				local color = when_text[1]
				if v.date:match('Today') or v.date:match('Tomorrow') then
					color = {0, 0.8, 0, 1}
				end
				when_text[#when_text+1] = color
				when_text[#when_text+1] = dat .. '\n    ' .. v.text .. '\n\n'
				already[v.text] = true
			end
		end
	end

	love.graphics.setFont(font)
	love.graphics.printf(when_text, FSIZE, 0, WRAP)

	love.graphics.setCanvas()
end


function love.mousemoved(x, y, dx, dy)
	if android and love.mouse.isDown(1) and dy ~= 0 then
		canv_off = canv_off + dy
		limiter()
	end
end


function love.draw()
	love.graphics.clear()

	if whx ~= 0 or why ~= 0 then
		love.graphics.print(whx .. ', ' .. why)
	end

	love.graphics.draw(canv, 0, canv_off, 0, scale.x, scale.y)
end


function love.load()
	android = (love.system.getOS() == 'Android')

	local dx, dy = love.graphics.getDimensions()
	if android then
		scale = { x = math.min(dx, dy) / IWIDTH, y = math.min(dx, dy) / IWIDTH }
		-- local aspect = math.max(dx, dy) / math.min(dx, dy)
	end

	local items = parseCalendar()
	local date = tonumber(os.time())
	whenq = when:calendar(items, date, FUTURE)

	canv_ht = FSIZE * get_ht() * 3 + FSIZE
	canv = love.graphics.newCanvas(IWIDTH, canv_ht, { dpiscale = 1 })
	font = love.graphics.newFont(FONT_FILE, FSIZE)
	canv_ht = canv_ht - dy
end


function love.update(dt)
	time = time + dt
	if time > 1 then
		print_cal()
	end
end


function love.wheelmoved(x, y)
	if not android and y ~= 0 then
		canv_off = canv_off + y * FSIZE
		limiter()
	end
end
